
#include "SD_card.hpp"

#include <stdlib.h>                 // atoi()
#include <stdio.h>                  // printf() or iprintf()
#include <string.h>                 // strtok()
#include <CString.hpp>
#include "CS.h"
#include "osHandles.h"
#include "fat/ff.h"
#include "fat/diskio.h"
#include "utils/rprintf.h"
#include "drivers/uart0.h"
#include "drivers/sta013.h"
#include "drivers/pcm1774.h"
#include "drivers/TSOP1236.h"
#include "utils/vector.hpp"


#define sdDetected	!(IOPIN0 & (1 << 16))
#define wrProtected !(IOPIN0 & (1 << 21))

void sdTask(void *sd)
{
	OSHANDLES *osHandles = (OSHANDLES*)sd;
	while(1)
	{
		int sdFlag;
		if(xQueueReceive(osHandles->queue.ctrlToSD, &sdFlag, portMAX_DELAY) )
		{
			if(sdFlag == 0)
			{
				if(!sdDetected){
					rprintf("SD card NOT inserted, please insert & press PLAY button again\n");
					continue;
				}
				DIR Dir;
				FILINFO Finfo;
				FRESULT returnCode = FR_OK;
				unsigned int fileBytesTotal, numFiles, numDirs;
				fileBytesTotal = numFiles = numDirs = 0;
				#if _USE_LFN
				char Lfname[512];
				#endif

				char dirPath[] = "0:";
				if (FR_OK != (returnCode = f_opendir(&Dir, dirPath)))
					continue;
				for (;;)
				{
					#if _USE_LFN
					Finfo.lfname = Lfname;
					Finfo.lfsize = sizeof(Lfname);
					#endif

					char returnCode = f_readdir(&Dir, &Finfo);
					if ( (FR_OK != returnCode) || !Finfo.fname[0])
						break;
					if (Finfo.fattrib & AM_DIR)
						numDirs++;
					else
					{
						numFiles++;
						fileBytesTotal += Finfo.fsize;
					}
					if (returnCode!= FR_OK || Finfo.fname[0] == 0)
						break;  /* Break on error or end of dir */
					else
					{
						 cString songName = &Finfo.fname[0];
						 if(songName.endsWith(".mp3") == 0)
						 {
							 rprintf("Sent %s to songNameQueue\n", &songName[0]);
							 xQueueSend(osHandles->queue.songNameQueue, &songName[0], (6*1000/portTICK_RATE_MS));
							 rprintf("Queue Send Complete\n");
						 }
						 else
							rprintf("No File Name with .mp3 Extension Found\n");//error msg if no .mp3 songs on SD card
					}
				}
				cString done = "done";
				rprintf("Sent %s to songNameQueue\n", &done[0]);
				xQueueSend(osHandles->queue.songNameQueue, &done[0], (6*1000/portTICK_RATE_MS));
				rprintf("Queue Send Complete\n");
			}
		}
	}
}
